Stan Math Library: stan/math/prim/scal/prob/pareto_type_2_cdf_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

pareto_type_2_cdf_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_PARETO_TYPE_2_CDF_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_PARETO_TYPE_2_CDF_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_finite.hpp>
9 #include <stan/math/prim/scal/err/check_greater_or_equal.hpp>
10 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
11 #include <stan/math/prim/scal/err/check_not_nan.hpp>
12 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
13 #include <stan/math/prim/scal/fun/constants.hpp>
14 #include <stan/math/prim/scal/fun/value_of.hpp>
15 #include <stan/math/prim/scal/fun/log1m.hpp>
16 #include <stan/math/prim/scal/meta/VectorView.hpp>
17 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
18 #include <boost/random/variate_generator.hpp>
19 #include <cmath>
20
21
22 namespace stan {
23 namespace math {
24
25 template <typename T_y, typename T_loc, typename T_scale, typename T_shape>
26 typename return_type<T_y, T_loc, T_scale, T_shape>::type
27 pareto_type_2_cdf_log(const T_y& y, const T_loc& mu,
28 const T_scale& lambda, const T_shape& alpha) {
29 typedef
30 typename stan::partials_return_type<T_y, T_loc, T_scale, T_shape>::type
31 T_partials_return;
32
33 // Check sizes
34 // Size checks
35 if ( !( stan::length(y)
36 && stan::length(mu)
37 && stan::length(lambda)
38 && stan::length(alpha) ) )
39 return 0.0;
40
41 // Check errors
42 static const char* function("stan::math::pareto_type_2_cdf_log");
43
44 using stan::math::check_greater_or_equal;
45 using stan::math::check_finite;
46 using stan::math::check_positive_finite;
47 using stan::math::check_not_nan;
48 using stan::math::check_greater_or_equal;
49 using stan::math::check_consistent_sizes;
50 using stan::math::check_nonnegative;
51 using stan::math::value_of;
52 using stan::math::log1m;
53 using std::log;
54
55 T_partials_return P(0.0);
56
57 check_greater_or_equal(function, "Random variable", y, mu);
58 check_not_nan(function, "Random variable", y);
59 check_nonnegative(function, "Random variable", y);
60 check_positive_finite(function, "Scale parameter", lambda);
61 check_positive_finite(function, "Shape parameter", alpha);
62 check_consistent_sizes(function,
63 "Random variable", y,
64 "Scale parameter", lambda,
65 "Shape parameter", alpha);
66
67 // Wrap arguments in vectors
68 VectorView<const T_y> y_vec(y);
69 VectorView<const T_loc> mu_vec(mu);
70 VectorView<const T_scale> lambda_vec(lambda);
71 VectorView<const T_shape> alpha_vec(alpha);
72 size_t N = max_size(y, mu, lambda, alpha);
73
74 OperandsAndPartials<T_y, T_loc, T_scale, T_shape>
75 operands_and_partials(y, mu, lambda, alpha);
76
77 VectorBuilder<true, T_partials_return,
78 T_y, T_loc, T_scale, T_shape>
79 cdf_log(N);
80
81 VectorBuilder<true, T_partials_return,
82 T_y, T_loc, T_scale, T_shape>
83 inv_p1_pow_alpha_minus_one(N);
84
85 VectorBuilder<!is_constant_struct<T_shape>::value,
86 T_partials_return, T_y, T_loc, T_scale, T_shape>
87 log_1p_y_over_lambda(N);
88
89 for (size_t i = 0; i < N; i++) {
90 const T_partials_return temp = 1.0 + (value_of(y_vec[i])
91 - value_of(mu_vec[i]))
92 / value_of(lambda_vec[i]);
93 const T_partials_return p1_pow_alpha
94 = pow(temp, value_of(alpha_vec[i]));
95 cdf_log[i] = log1m(1.0 / p1_pow_alpha);
96
97 inv_p1_pow_alpha_minus_one[i] = 1.0 / (p1_pow_alpha - 1.0);
98
99 if (!is_constant_struct<T_shape>::value)
100 log_1p_y_over_lambda[i] = log(temp);
101 }
102
103 // Compute vectorized CDF and its gradients
104
105 for (size_t n = 0; n < N; n++) {
106 // Pull out values
107 const T_partials_return y_dbl = value_of(y_vec[n]);
108 const T_partials_return mu_dbl = value_of(mu_vec[n]);
109 const T_partials_return lambda_dbl = value_of(lambda_vec[n]);
110 const T_partials_return alpha_dbl = value_of(alpha_vec[n]);
111
112 const T_partials_return grad_1_2 = alpha_dbl
113 * inv_p1_pow_alpha_minus_one[n] / (lambda_dbl - mu_dbl + y_dbl);
114
115 // Compute
116 P += cdf_log[n];
117
118 if (!is_constant_struct<T_y>::value)
119 operands_and_partials.d_x1[n] += grad_1_2;
120 if (!is_constant_struct<T_loc>::value)
121 operands_and_partials.d_x2[n] -= grad_1_2;
122 if (!is_constant_struct<T_scale>::value)
123 operands_and_partials.d_x3[n] += (mu_dbl - y_dbl) * grad_1_2
124 / lambda_dbl;
125 if (!is_constant_struct<T_shape>::value)
126 operands_and_partials.d_x4[n] += log_1p_y_over_lambda[n]
127 * inv_p1_pow_alpha_minus_one[n];
128 }
129
130 return operands_and_partials.value(P);
131 }
132 }
133 }
134 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::check_greater_or_equal
bool check_greater_or_equal(const char *function, const char *name, const T_y &y, const T_low &low)
Return true if y is greater or equal than low.
Definition: check_greater_or_equal.hpp:84
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
VectorView.hpp
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
log1m.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
stan::math::check_finite
bool check_finite(const char *function, const char *name, const T_y &y)
Return true if y is finite.
Definition: check_finite.hpp:62
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
check_finite.hpp
stan::math::pareto_type_2_cdf_log
return_type< T_y, T_loc, T_scale, T_shape >::type pareto_type_2_cdf_log(const T_y &y, const T_loc &mu, const T_scale &lambda, const T_shape &alpha)
Definition: pareto_type_2_cdf_log.hpp:27
stan::math::pow
fvar< T > pow(const fvar< T > &x1, const fvar< T > &x2)
Definition: pow.hpp:18
stan::math::check_nonnegative
bool check_nonnegative(const char *function, const char *name, const T_y &y)
Return true if y is non-negative.
Definition: check_nonnegative.hpp:66
check_greater_or_equal.hpp
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::log1m
fvar< T > log1m(const fvar< T > &x)
Definition: log1m.hpp:16
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
stan::math::OperandsAndPartials::d_x4
VectorView< T_return_type, false, true > d_x4
Definition: OperandsAndPartials.hpp:42

[ Stan Home Page ] © 2011–2016, Stan Development Team.